#include  <iostream>
#include  <vector>
#include <fastjet/PseudoJet.hh>
#include <fastjet/ClusterSequence.hh>
using namespace fastjet;
using namespace std;

const int mxpart = 12;
const int maxd   = 40;

extern "C" {

// integer nextnplot
// common/plotindex/nextnplot
extern struct {
  int nextnplot;
} plotindex_;


// integer maxd,ndmax
// parameter (maxd=40)
// double precision ptilde(0:maxd,mxpart,4)
// double precision ptildejet(0:maxd,mxpart,4)
// common/ptildes/ptilde,ptildejet,ndmax
extern struct {
  double ptilde[4][mxpart][maxd+1];
  double ptildejet[4][mxpart][maxd+1];
  int ndmax;
} ptildes_;


// integer npart
// common/npart/npart
extern struct {
  int npart;
} npart_;


// character*2 plabel(mxpart)
// common/plabel/plabel
extern struct {
  char plabel[mxpart][2];
} plabel_;


void bookplot_(const int & iplot,  //< the index of the plot 
               const char * tag,   //< either "book" or "plot"
               const char * title, //< the plot title
               const double & var,
               const double & wt,
               const double & wt2,
               const double & xmax,
               const double & xmin,
               const double & dx,
               const char * llplot, // "lin" or "log"
               const int tag_len,   // pass string lengths by value!
               const int title_len  // pass string lengths by value!
               );

// SSbegin
// double precision facscale 
// common/facscale/facscale
extern struct {
  double facscale;
} facscale_;

// double precision dipscale(0:maxd)
// common/dipolescale/dipscale
extern struct {
  double dipscale[maxd+1];
} dipolescale_;

// common/part/part
extern struct {
  char part[4];
} part_;

// common/useropt/purevirt
extern struct {
  bool purevirt;
} useropt_;

// common/reweight/reweight
extern struct {
  double reweight;
} reweight_;



void mcfmmain_(const char * inputfile,
               const char * workdir);

void mcfmfwrite_(const int  & unitno,
                 const char * string,
                 const int string_length);

// SSend

} // extern "C"

//----------------------------------------------------------------------
/// Interface to carry out histogramming with the MCFM mbook package.
///
/// When calling from C++, use this interface and not the f77
/// bookplot_ one defined above, because this one correctly handles the
/// the string aspects
inline void bookplot( const int & iplot,         //< the index of the plot 
               const std::string & tag,   //< either "book" or "plot"
               const std::string & title, //< the plot title
               const double & var,        //< the value of the variable
               const double & wt,         //< event weight
               const double & wt2,        //< squared event weight
               const double & xmax,       //< lower limit of histogram
               const double & xmin,       //< upper limit of histogram
               const double & dx,         //< bin spacing
               const std::string & llplot // "lin" or "log"
               ) {
  bookplot_(iplot, tag.c_str(), title.c_str(), 
            var, wt, wt2, xmax, xmin, dx, llplot.c_str(), 
            tag.size(),
            title.size());

}


//----------------------------------------------------------------------
/// returns a vector of the particles in the event; if the offset
/// is zero, then it corresponds to all outgoing particles; 
///
/// if the offset is higher, then the first "offset" particles are
/// discarded;
///
/// if the offset is -2, then the incoming particles are the first two
///
inline vector<PseudoJet> mcfm_particles(int nd, int offset = 0) {
  vector<PseudoJet> result(npart_.npart-offset);
  for (int i = 2 + offset; i < 2+npart_.npart; i++) {
    result[i-(2+offset)].reset(ptildes_.ptilde[0][i][nd],
                               ptildes_.ptilde[1][i][nd],
                               ptildes_.ptilde[2][i][nd],
                               ptildes_.ptilde[3][i][nd]);
  }
  return result;
}

//----------------------------------------------------------------------
/// interface to write a C string to a fortran unit
inline void mcfmfwrite(int unitno,
                const std::string & string) {
  mcfmfwrite_(unitno, string.c_str(), string.size());
}

//----------------------------------------------------------------------
/// interface to write a C string to a fortran unit, preceded by two
/// comment characters (in the format that is present in userwriteinfo_)
inline void mcfmfwrite(int unitno,
                const char * comment_chars, //< assumed to be a length 2 array
                const std::string & string) {
  std::string newstring = comment_chars[0] + string + comment_chars[1];
  mcfmfwrite(unitno, newstring.c_str());
}
